@@ -4,8 +4,8 @@ import enum
 from sqlalchemy import Column, Float, String
 from sqlalchemy.ext.declarative import declarative_base
 
-from zvdata import Mixin
-from zvdata.contract import register_schema
+from zvt.contract import Mixin
+from zvt.contract.register import register_schema
 
 Stock1dZenFactorBase = declarative_base()
 
