@@ -2,19 +2,55 @@
 
 import pandas as pd
 
-from zvdata.utils.pd_utils import normal_index_df
-from zvt.api.computing import macd
 from zvt.factors.factor import Scorer, Transformer
+from zvt.utils.pd_utils import normal_index_df
+
+
+def ma(s, window=5):
+    """
+
+    :param s:
+    :type s:pd.Series
+    :param window:
+    :type window:
+    :return:
+    :rtype:
+    """
+    return s.rolling(window=window, min_periods=window).mean()
+
+
+def ema(s, window=12):
+    return s.ewm(span=window, adjust=False, min_periods=window).mean()
+
+
+def macd(s, slow=26, fast=12, n=9, return_type='se'):
+    ema_fast = ema(s, window=fast)
+
+    ema_slow = ema(s, window=slow)
+
+    diff = ema_fast - ema_slow
+    dea = diff.ewm(span=n, adjust=False).mean()
+    m = (diff - dea) * 2
+
+    if return_type == 'se':
+        return diff, dea, m
+    else:
+        return pd.DataFrame({'diff': diff, 'dea': dea, 'macd': m})
 
 
 class MaTransformer(Transformer):
-    def __init__(self, windows=[5, 10]) -> None:
+    def __init__(self, windows=[5, 10], cal_change_pct=False) -> None:
+        super().__init__()
         self.windows = windows
+        self.cal_change_pct = cal_change_pct
 
     def transform(self, input_df) -> pd.DataFrame:
+        if self.cal_change_pct:
+            input_df['change_pct'] = input_df['close'].pct_change()
+
         for window in self.windows:
             col = 'ma{}'.format(window)
-            self.indicator_cols.append(col)
+            self.indicators.append(col)
 
             ma_df = input_df['close'].groupby(level=0).rolling(window=window, min_periods=window).mean()
             ma_df = ma_df.reset_index(level=0, drop=True)
@@ -25,13 +61,14 @@ class MaTransformer(Transformer):
 
 class MaAndVolumeTransformer(Transformer):
     def __init__(self, windows=[5, 10], vol_windows=[30]) -> None:
+        super().__init__()
         self.windows = windows
         self.vol_windows = vol_windows
 
     def transform(self, input_df) -> pd.DataFrame:
         for window in self.windows:
             col = 'ma{}'.format(window)
-            self.indicator_cols.append(col)
+            self.indicators.append(col)
 
             ma_df = input_df['close'].groupby(level=0).rolling(window=window, min_periods=window).mean()
             ma_df = ma_df.reset_index(level=0, drop=True)
@@ -39,7 +76,7 @@ class MaAndVolumeTransformer(Transformer):
 
         for vol_window in self.vol_windows:
             col = 'vol_ma{}'.format(vol_window)
-            self.indicator_cols.append(col)
+            self.indicators.append(col)
 
             vol_ma_df = input_df['volume'].groupby(level=0).rolling(window=vol_window, min_periods=vol_window).mean()
             vol_ma_df = vol_ma_df.reset_index(level=0, drop=True)
@@ -50,13 +87,14 @@ class MaAndVolumeTransformer(Transformer):
 
 class MacdTransformer(Transformer):
     def __init__(self, slow=26, fast=12, n=9) -> None:
+        super().__init__()
         self.slow = slow
         self.fast = fast
         self.n = n
 
-        self.indicator_cols.append('diff')
-        self.indicator_cols.append('dea')
-        self.indicator_cols.append('macd')
+        self.indicators.append('diff')
+        self.indicators.append('dea')
+        self.indicators.append('macd')
 
     def transform(self, input_df) -> pd.DataFrame:
         macd_df = input_df.groupby(level=0)['close'].apply(
