@@ -2,47 +2,55 @@
 
 import logging
 import math
+from typing import List
 
-from zvdata.contract import get_db_session
-from zvdata import IntervalLevel
-from zvt.api.business import get_account
-from zvt.api.common import decode_entity_id, get_kdata_schema
-from zvt.api.rules import get_trading_meta
-from zvt.api.quote import get_kdata
-from zvt.domain import Order
-from zvt.domain.business import SimAccount, Position
+from zvt.api import get_kdata
+from zvt.api.business import get_trader_info
+from zvt.api.quote import decode_entity_id, get_kdata_schema
+from zvt.contract import IntervalLevel, EntityMixin
+from zvt.contract.api import get_db_session
+from zvt.domain.trader_info import AccountStats, Position, Order, TraderInfo
 from zvt.trader import TradingSignalType, TradingListener, TradingSignal
-from zvt.trader.errors import NotEnoughMoneyError, InvalidOrderError, NotEnoughPositionError, InvalidOrderParamError
-from zvdata.utils.time_utils import to_pd_timestamp, to_time_str, TIME_FORMAT_ISO8601, is_same_date
-from zvdata.utils.utils import fill_domain_from_dict
+from zvt.trader.errors import NotEnoughMoneyError, InvalidOrderError, NotEnoughPositionError, InvalidOrderParamError, \
+    WrongKdataError
+from zvt.utils.pd_utils import pd_is_not_null
+from zvt.utils.time_utils import to_pd_timestamp, to_time_str, TIME_FORMAT_ISO8601, is_same_date
+from zvt.utils.utils import fill_domain_from_dict
 
 ORDER_TYPE_LONG = 'order_long'
 ORDER_TYPE_SHORT = 'order_short'
 ORDER_TYPE_CLOSE_LONG = 'order_close_long'
 ORDER_TYPE_CLOSE_SHORT = 'order_close_short'
 
-from marshmallow_sqlalchemy import ModelSchema
+from marshmallow_sqlalchemy import SQLAlchemyAutoSchema
 
 
-class SimAccountSchema(ModelSchema):
+# FIXME:better way for schema<->domain,now just dump to schema and use dict['field'] for operation
+class AccountDayStatsSchema(SQLAlchemyAutoSchema):
     class Meta:
-        model = SimAccount
+        model = AccountStats
+        include_relationships = True
 
 
-class PositionSchema(ModelSchema):
+class PositionSchema(SQLAlchemyAutoSchema):
     class Meta:
         model = Position
+        include_relationships = True
 
 
-sim_account_schema = SimAccountSchema()
+account_stats_schema = AccountDayStatsSchema()
 position_schema = PositionSchema()
 
 
 class AccountService(TradingListener):
     logger = logging.getLogger(__name__)
-    trader_name = None
 
     def get_current_position(self, entity_id):
+        """
+        overwrite it to provide your real position
+
+        :param entity_id:
+        """
         pass
 
     def order(self, entity_id, current_price, current_timestamp, order_amount=0, order_pct=1.0, order_price=0,
@@ -75,153 +83,178 @@ class AccountService(TradingListener):
 
     @staticmethod
     def trading_signal_to_order_type(trading_signal_type):
-        if trading_signal_type == TradingSignalType.trading_signal_open_long:
+        if trading_signal_type == TradingSignalType.open_long:
             return ORDER_TYPE_LONG
-        if trading_signal_type == TradingSignalType.trading_signal_open_short:
+        if trading_signal_type == TradingSignalType.open_short:
             return ORDER_TYPE_SHORT
-        if trading_signal_type == TradingSignalType.trading_signal_close_long:
+        if trading_signal_type == TradingSignalType.close_long:
             return ORDER_TYPE_CLOSE_LONG
-        if trading_signal_type == TradingSignalType.trading_signal_close_short:
+        if trading_signal_type == TradingSignalType.close_short:
             return ORDER_TYPE_CLOSE_SHORT
 
-    def on_trading_signal(self, trading_signal: TradingSignal):
-        self.logger.debug('trader:{} received trading signal:{}'.format(self.trader_name, trading_signal))
-        entity_id = trading_signal.entity_id
-        current_timestamp = trading_signal.the_timestamp
-        order_type = AccountService.trading_signal_to_order_type(trading_signal.trading_signal_type)
-        trading_level = trading_signal.trading_level.value
-        if order_type:
-            try:
-                kdata = get_kdata(provider=self.provider, entity_id=entity_id, level=trading_level,
-                                  start_timestamp=current_timestamp, end_timestamp=current_timestamp,
-                                  limit=1)
-                if kdata is not None and not kdata.empty:
-                    entity_type, _, _ = decode_entity_id(kdata['entity_id'][0])
-
-                    the_price = kdata['close'][0]
-
-                    if the_price:
-                        self.order(entity_id=entity_id, current_price=the_price,
-                                   current_timestamp=current_timestamp, order_pct=trading_signal.position_pct,
-                                   order_money=trading_signal.order_money,
-                                   order_type=order_type)
-                    else:
-                        self.logger.warning(
-                            'ignore trading signal,wrong kdata,entity_id:{},timestamp:{},kdata:{}'.format(entity_id,
-                                                                                                          current_timestamp,
-                                                                                                          kdata.to_dict(
-                                                                                                              orient='records')))
-
-                else:
-                    self.logger.warning(
-                        'ignore trading signal,could not get kdata,entity_id:{},timestamp:{}'.format(entity_id,
-                                                                                                     current_timestamp))
-            except Exception as e:
-                self.logger.exception(e)
-
 
 class SimAccountService(AccountService):
 
-    def __init__(self, trader_name,
+    def __init__(self,
+                 entity_schema: EntityMixin,
+                 trader_name,
                  timestamp,
-                 provider='joinquant',
+                 provider=None,
                  level=IntervalLevel.LEVEL_1DAY,
                  base_capital=1000000,
                  buy_cost=0.001,
                  sell_cost=0.001,
                  slippage=0.001):
-
+        self.entity_schema = entity_schema
         self.base_capital = base_capital
         self.buy_cost = buy_cost
         self.sell_cost = sell_cost
         self.slippage = slippage
         self.trader_name = trader_name
 
-        self.session = get_db_session('zvt', 'business')
+        self.session = get_db_session('zvt', data_schema=TraderInfo)
         self.provider = provider
         self.level = level
         self.start_timestamp = timestamp
 
-        account = get_account(session=self.session, trader_name=self.trader_name, return_type='domain', limit=1)
+        self.account: AccountStats = self.init_account()
 
-        if account:
-            self.logger.warning("trader:{} has run before,old result would be deleted".format(trader_name))
-            self.session.query(SimAccount).filter(SimAccount.trader_name == self.trader_name).delete()
+    def init_account(self) -> AccountStats:
+        trader_info = get_trader_info(session=self.session, trader_name=self.trader_name, return_type='domain',
+                                      limit=1)
+
+        if trader_info:
+            self.logger.warning("trader:{} has run before,old result would be deleted".format(self.trader_name))
+            self.session.query(TraderInfo).filter(TraderInfo.trader_name == self.trader_name).delete()
+            self.session.query(AccountStats).filter(AccountStats.trader_name == self.trader_name).delete()
             self.session.query(Position).filter(Position.trader_name == self.trader_name).delete()
             self.session.query(Order).filter(Order.trader_name == self.trader_name).delete()
             self.session.commit()
 
-        account = SimAccount(trader_name=self.trader_name, cash=self.base_capital,
-                             positions=[], all_value=self.base_capital, value=0, closing=False,
-                             timestamp=timestamp)
-        self.latest_account = sim_account_schema.dump(account)
+        return AccountStats(entity_id=f'trader_zvt_{self.trader_name}',
+                            timestamp=self.start_timestamp,
+                            trader_name=self.trader_name,
+                            cash=self.base_capital,
+                            all_value=self.base_capital,
+                            value=0,
+                            closing=False
+                            )
 
-        # self.persist_account(timestamp)
+    def load_account(self) -> AccountStats:
+        records = AccountStats.query_data(filters=[AccountStats.trader_name == self.trader_name],
+                                          order=AccountStats.timestamp.desc(), limit=1, return_type='domain')
+        latest_record: AccountStats = records[0]
+
+        # create new orm object from latest record
+        account_dict = account_stats_schema.dump(latest_record)
+        del account_dict['id']
+        del account_dict['positions']
+        account = AccountStats()
+        fill_domain_from_dict(account, account_dict)
+
+        positions: List[Position] = []
+        for position_domain in latest_record.positions:
+            position_dict = position_schema.dump(position_domain)
+            self.logger.info('current position:{}'.format(position_dict))
+            del position_dict['id']
+            del position_dict['account_stats']
+            position = Position()
+            fill_domain_from_dict(position, position_dict)
+            positions.append(position)
+
+        account.positions = positions
+
+        return account
 
     def on_trading_open(self, timestamp):
         self.logger.info('on_trading_open:{}'.format(timestamp))
         if is_same_date(timestamp, self.start_timestamp):
             return
-        # get the account for trading at the date
-        accounts = get_account(session=self.session, trader_name=self.trader_name, return_type='domain',
-                               end_timestamp=to_time_str(timestamp), limit=1, order=SimAccount.timestamp.desc())
-        if accounts:
-            account = accounts[0]
-        else:
-            return
+        self.account = self.load_account()
+        self.logger.info('on_trading_open:{},current_account:{}'.format(timestamp, self.account))
 
-        positions = []
-        # FIXME:dump all directly
-        for position_domain in account.positions:
-            position_dict = position_schema.dump(position_domain)
-            self.logger.info('current position:{}'.format(position_dict))
-            del position_dict['sim_account']
-            positions.append(position_dict)
+    def on_trading_error(self, timestamp, error):
+        pass
 
-        self.latest_account = sim_account_schema.dump(account)
-        self.latest_account['positions'] = positions
-        self.logger.info('on_trading_open:{},latest_account:{}'.format(timestamp, self.latest_account))
+    def on_trading_finish(self, timestamp):
+        pass
+
+    def on_trading_signal(self, trading_signal: TradingSignal):
+        entity_id = trading_signal.entity_id
+        happen_timestamp = trading_signal.happen_timestamp
+        order_type = AccountService.trading_signal_to_order_type(trading_signal.trading_signal_type)
+        trading_level = trading_signal.trading_level.value
+        if order_type:
+            try:
+                kdata = get_kdata(provider=self.provider, entity_id=entity_id, level=trading_level,
+                                  start_timestamp=happen_timestamp, end_timestamp=happen_timestamp,
+                                  limit=1)
+            except Exception as e:
+                self.logger.error(e)
+                raise WrongKdataError("could not get kdata")
+
+            if pd_is_not_null(kdata):
+                entity_type, _, _ = decode_entity_id(kdata['entity_id'][0])
+
+                the_price = kdata['close'][0]
+
+                if the_price:
+                    self.order(entity_id=entity_id, current_price=the_price,
+                               current_timestamp=happen_timestamp, order_pct=trading_signal.position_pct,
+                               order_money=trading_signal.order_money,
+                               order_type=order_type)
+                else:
+                    self.logger.warning(
+                        'ignore trading signal,wrong kdata,entity_id:{},timestamp:{},kdata:{}'.format(entity_id,
+                                                                                                      happen_timestamp,
+                                                                                                      kdata.to_dict(
+                                                                                                          orient='records')))
+
+            else:
+                self.logger.warning(
+                    'ignore trading signal,could not get kdata,entity_id:{},timestamp:{}'.format(entity_id,
+                                                                                                 happen_timestamp))
 
     def on_trading_close(self, timestamp):
         self.logger.info('on_trading_close:{}'.format(timestamp))
 
-        self.latest_account['value'] = 0
-        self.latest_account['all_value'] = 0
-        for position in self.latest_account['positions']:
-            entity_type, _, _ = decode_entity_id(position['entity_id'])
+        self.account.value = 0
+        self.account.all_value = 0
+        for position in self.account.positions:
+            entity_type, _, _ = decode_entity_id(position.entity_id)
             data_schema = get_kdata_schema(entity_type, level=self.level)
 
-            kdata = get_kdata(provider=self.provider, level=self.level, entity_id=position['entity_id'],
+            kdata = get_kdata(provider=self.provider, level=IntervalLevel.LEVEL_1DAY, entity_id=position.entity_id,
                               order=data_schema.timestamp.desc(),
                               end_timestamp=timestamp, limit=1)
 
             closing_price = kdata['close'][0]
 
-            position['available_long'] = position['long_amount']
-            position['available_short'] = position['short_amount']
+            position.available_long = position.long_amount
+            position.available_short = position.short_amount
 
             if closing_price:
-                if (position['long_amount'] is not None) and position['long_amount'] > 0:
-                    position['value'] = position['long_amount'] * closing_price
-                    self.latest_account['value'] += position['value']
-                elif (position['short_amount'] is not None) and position['short_amount'] > 0:
-                    position['value'] = 2 * (position['short_amount'] * position['average_short_price'])
-                    position['value'] -= position['short_amount'] * closing_price
-                    self.latest_account['value'] += position['value']
+                if (position.long_amount is not None) and position.long_amount > 0:
+                    position.value = position.long_amount * closing_price
+                    self.account.value += position.value
+                elif (position.short_amount is not None) and position.short_amount > 0:
+                    position.value = 2 * (position.short_amount * position.average_short_price)
+                    position.value -= position.short_amount * closing_price
+                    self.account.value += position.value
             else:
                 self.logger.warning(
                     'could not refresh close value for position:{},timestamp:{}'.format(position['entity_id'],
                                                                                         timestamp))
 
         # remove the empty position
-        self.latest_account['positions'] = [position for position in self.latest_account['positions'] if
-                                            position['long_amount'] > 0 or position['short_amount'] > 0]
+        self.account.positions = [position for position in self.account.positions if
+                                  position.long_amount > 0 or position.short_amount > 0]
 
-        self.latest_account['all_value'] = self.latest_account['value'] + self.latest_account['cash']
-        self.latest_account['closing'] = True
-        self.latest_account['timestamp'] = to_pd_timestamp(timestamp)
+        self.account.all_value = self.account.value + self.account.cash
+        self.account.closing = True
+        self.account.timestamp = to_pd_timestamp(timestamp)
 
-        self.logger.info('on_trading_close:{},latest_account:{}'.format(timestamp, self.latest_account))
+        self.logger.info('on_trading_close:{},latest_account:{}'.format(timestamp, self.account))
         self.persist_account(timestamp)
 
     def persist_account(self, timestamp):
@@ -232,53 +265,34 @@ class SimAccountService(AccountService):
         :type timestamp:
         """
         the_id = '{}_{}'.format(self.trader_name, to_time_str(timestamp, TIME_FORMAT_ISO8601))
-        positions = []
-        for position in self.latest_account['positions']:
-            position_domain = Position()
-            fill_domain_from_dict(position_domain, position, None)
 
-            position_domain.id = '{}_{}_{}'.format(self.trader_name, position['entity_id'],
-                                                   to_time_str(timestamp, TIME_FORMAT_ISO8601))
-            position_domain.timestamp = to_pd_timestamp(timestamp)
-            position_domain.sim_account_id = the_id
+        for position in self.account.positions:
+            position.id = '{}_{}_{}'.format(self.trader_name, position.entity_id,
+                                            to_time_str(timestamp, TIME_FORMAT_ISO8601))
+            position.timestamp = to_pd_timestamp(timestamp)
+            position.account_stats_id = the_id
 
-            positions.append(position_domain)
+        self.account.id = the_id
 
-        account_domain = SimAccount(id=the_id, trader_name=self.trader_name, cash=self.latest_account['cash'],
-                                    positions=positions,
-                                    all_value=self.latest_account['all_value'], value=self.latest_account['value'],
-                                    timestamp=to_pd_timestamp(self.latest_account['timestamp']))
+        self.logger.info('persist_account:{}'.format(account_stats_schema.dump(self.account)))
 
-        self.logger.info('persist_account:{}'.format(sim_account_schema.dump(account_domain)))
-
-        self.session.add(account_domain)
+        self.session.add(self.account)
         self.session.commit()
 
-    def get_current_position(self, entity_id):
+    def get_current_position(self, entity_id) -> Position:
         """
-        get current position to design whether order could make
+        get current position to decide whether order could make
 
         :param entity_id:
         :type entity_id: str
         :return:
         :rtype: None
         """
-        for position in self.latest_account['positions']:
-            if position['entity_id'] == entity_id:
+        for position in self.account.positions:
+            if position.entity_id == entity_id:
                 return position
         return None
 
-    def get_account_at_time(self, timestamp):
-        """
-
-        :param timestamp:
-        :type timestamp:
-        :return:
-        :rtype:SimAccount
-        """
-        return get_account(session=self.session, trader_name=self.trader_name, return_type='domain',
-                           end_timestamp=timestamp, limit=1)[0]
-
     def update_position(self, current_position, order_amount, current_price, order_type, timestamp):
         """
 
@@ -295,57 +309,61 @@ class SimAccountService(AccountService):
         """
         if order_type == ORDER_TYPE_LONG:
             need_money = (order_amount * current_price) * (1 + self.slippage + self.buy_cost)
-            if self.latest_account['cash'] < need_money:
+            if self.account.cash < need_money:
                 raise NotEnoughMoneyError()
 
-            self.latest_account['cash'] -= need_money
+            self.account.cash -= need_money
 
             # 计算平均价
-            long_amount = current_position['long_amount'] + order_amount
-            current_position['average_long_price'] = (current_position['average_long_price'] * current_position[
-                'long_amount'] + current_price * order_amount) / long_amount
+            long_amount = current_position.long_amount + order_amount
+            current_position.average_long_price = (current_position.average_long_price * current_position.long_amount
+                                                   + current_price * order_amount) / long_amount
 
-            current_position['long_amount'] = long_amount
+            current_position.long_amount = long_amount
 
-            if current_position['trading_t'] == 0:
-                current_position['available_long'] += order_amount
+            if current_position.trading_t == 0:
+                current_position.available_long += order_amount
 
         elif order_type == ORDER_TYPE_SHORT:
             need_money = (order_amount * current_price) * (1 + self.slippage + self.buy_cost)
-            if self.latest_account['cash'] < need_money:
-                raise NotEnoughMoneyError
+            if self.account.cash < need_money:
+                raise NotEnoughMoneyError()
 
-            self.latest_account['cash'] -= need_money
+            self.account.cash -= need_money
 
-            short_amount = current_position['short_amount'] + order_amount
-            current_position['average_short_price'] = (current_position['average_short_price'] * current_position[
-                'short_amount']
-                                                       + current_price * order_amount) / short_amount
+            short_amount = current_position.short_amount + order_amount
+            current_position.average_short_price = (current_position.average_short_price * current_position.short_amount
+                                                    + current_price * order_amount) / short_amount
 
-            current_position['short_amount'] = short_amount
+            current_position.short_amount = short_amount
 
-            if current_position['trading_t'] == 0:
-                current_position['available_short'] += order_amount
+            if current_position.trading_t == 0:
+                current_position.available_short += order_amount
 
         elif order_type == ORDER_TYPE_CLOSE_LONG:
-            self.latest_account['cash'] += (order_amount * current_price * (1 - self.slippage - self.sell_cost))
+            self.account.cash += (order_amount * current_price * (1 - self.slippage - self.sell_cost))
 
-            current_position['available_long'] -= order_amount
-            current_position['long_amount'] -= order_amount
+            current_position.available_long -= order_amount
+            current_position.long_amount -= order_amount
 
         elif order_type == ORDER_TYPE_CLOSE_SHORT:
-            self.latest_account['cash'] += 2 * (order_amount * current_position['average_short_price'])
-            self.latest_account['cash'] -= order_amount * current_price * (1 + self.slippage + self.sell_cost)
+            self.account.cash += 2 * (order_amount * current_position.average_short_price)
+            self.account.cash -= order_amount * current_price * (1 + self.slippage + self.sell_cost)
 
-            current_position['available_short'] -= order_amount
-            current_position['short_amount'] -= order_amount
+            current_position.available_short -= order_amount
+            current_position.short_amount -= order_amount
 
         # save the order info to db
-        order_id = '{}_{}_{}_{}'.format(self.trader_name, order_type, current_position['entity_id'],
+        order_id = '{}_{}_{}_{}'.format(self.trader_name, order_type, current_position.entity_id,
                                         to_time_str(timestamp, TIME_FORMAT_ISO8601))
-        order = Order(id=order_id, timestamp=to_pd_timestamp(timestamp), trader_name=self.trader_name,
-                      entity_id=current_position['entity_id'], order_price=current_price, order_amount=order_amount,
+        order = Order(id=order_id,
+                      timestamp=to_pd_timestamp(timestamp),
+                      trader_name=self.trader_name,
+                      entity_id=current_position.entity_id,
+                      order_price=current_price,
+                      order_amount=order_amount,
                       order_type=order_type,
+                      level=self.level.value,
                       status='success')
         self.session.add(order)
         self.session.commit()
@@ -388,31 +406,30 @@ class SimAccountService(AccountService):
             current_position = self.get_current_position(entity_id=entity_id)
 
             if not current_position:
-                trading_t = get_trading_meta(entity_id=entity_id)['trading_t']
-                current_position = {
-                    'trader_name': self.trader_name,
-                    'entity_id': entity_id,
-                    'long_amount': 0,
-                    'available_long': 0,
-                    'average_long_price': 0,
-                    'short_amount': 0,
-                    'available_short': 0,
-                    'average_short_price': 0,
-                    'profit': 0,
-                    'value': 0,
-                    'trading_t': trading_t
-                }
+                trading_t = self.entity_schema.get_trading_t()
+                current_position = Position(trader_name=self.trader_name,
+                                            entity_id=entity_id,
+                                            long_amount=0,
+                                            available_long=0,
+                                            average_long_price=0,
+                                            short_amount=0,
+                                            available_short=0,
+                                            average_short_price=0,
+                                            profit=0,
+                                            value=0,
+                                            trading_t=trading_t
+                                            )
                 # add it to latest account
-                self.latest_account['positions'].append(current_position)
+                self.account.positions.append(current_position)
 
             # 按钱交易
             if order_money > 0:
                 # 开多
                 if order_type == ORDER_TYPE_LONG:
-                    if current_position['short_amount'] > 0:
+                    if current_position.short_amount > 0:
                         raise InvalidOrderError("close the short position before open long")
 
-                    if order_money > self.latest_account['cash']:
+                    if order_money > self.account.cash:
                         raise NotEnoughMoneyError()
 
                     cost = current_price * (1 + self.slippage + self.buy_cost)
@@ -426,10 +443,10 @@ class SimAccountService(AccountService):
                         raise NotEnoughMoneyError()
                 # 开空
                 elif order_type == ORDER_TYPE_SHORT:
-                    if current_position['long_amount'] > 0:
+                    if current_position.long_amount > 0:
                         raise InvalidOrderError("close the long position before open short")
 
-                    if order_money > self.latest_account['cash']:
+                    if order_money > self.account.cash:
                         raise NotEnoughMoneyError()
 
                     cost = current_price * (1 + self.slippage + self.buy_cost)
@@ -447,26 +464,26 @@ class SimAccountService(AccountService):
             elif order_amount > 0:
                 # 开多
                 if order_type == ORDER_TYPE_LONG:
-                    if current_position['short_amount'] > 0:
+                    if current_position.short_amount > 0:
                         raise InvalidOrderError("close the short position before open long")
 
                     self.update_position(current_position, order_amount, current_price, order_type, current_timestamp)
                 # 开空
                 elif order_type == ORDER_TYPE_SHORT:
-                    if current_position['long_amount'] > 0:
+                    if current_position.long_amount > 0:
                         raise InvalidOrderError("close the long position before open short")
 
                     self.update_position(current_position, order_amount, current_price, order_type, current_timestamp)
                 # 平多
                 elif order_type == ORDER_TYPE_CLOSE_LONG:
-                    if current_position['available_long'] >= order_amount:
+                    if current_position.available_long >= order_amount:
                         self.update_position(current_position, order_amount, current_price, order_type,
                                              current_timestamp)
                     else:
                         raise NotEnoughPositionError()
                 # 平空
                 elif order_type == ORDER_TYPE_CLOSE_SHORT:
-                    if current_position['available_short'] >= order_amount:
+                    if current_position.available_short >= order_amount:
                         self.update_position(current_position, order_amount, current_price, order_type,
                                              current_timestamp)
                     else:
@@ -476,11 +493,11 @@ class SimAccountService(AccountService):
             elif 0 < order_pct <= 1:
                 # 开多
                 if order_type == ORDER_TYPE_LONG:
-                    if current_position['short_amount'] > 0:
+                    if current_position.short_amount > 0:
                         raise InvalidOrderError("close the short position before open long")
 
                     cost = current_price * (1 + self.slippage + self.buy_cost)
-                    want_pay = self.latest_account['cash'] * order_pct
+                    want_pay = self.account.cash * order_pct
                     # 买的数量
                     order_amount = want_pay // cost
                     if order_amount > 0:
@@ -490,11 +507,11 @@ class SimAccountService(AccountService):
                         raise NotEnoughMoneyError()
                 # 开空
                 elif order_type == ORDER_TYPE_SHORT:
-                    if current_position['long_amount'] > 0:
+                    if current_position.long_amount > 0:
                         raise InvalidOrderError("close the long position before open short")
 
                     cost = current_price * (1 + self.slippage + self.buy_cost)
-                    want_pay = self.latest_account['cash'] * order_pct
+                    want_pay = self.account.cash * order_pct
 
                     order_amount = want_pay // cost
                     if order_amount > 0:
@@ -505,33 +522,33 @@ class SimAccountService(AccountService):
 
                 # 平多
                 elif order_type == ORDER_TYPE_CLOSE_LONG:
-                    if current_position['available_long'] > 0:
+                    if current_position.available_long > 0:
                         if order_pct == 1.0:
-                            order_amount = current_position['available_long']
+                            order_amount = current_position.available_long
                         else:
-                            order_amount = math.floor(current_position['available_long'] * order_pct)
+                            order_amount = math.floor(current_position.available_long * order_pct)
 
                         if order_amount != 0:
                             self.update_position(current_position, order_amount, current_price, order_type,
                                                  current_timestamp)
                         else:
                             self.logger.warning("{} available_long:{} order_pct:{} order_amount:{}", entity_id,
-                                                current_position['available_long'], order_pct, order_amount)
+                                                current_position.available_long, order_pct, order_amount)
                     else:
                         raise NotEnoughPositionError()
                 # 平空
                 elif order_type == ORDER_TYPE_CLOSE_SHORT:
-                    if current_position['available_short'] > 0:
+                    if current_position.available_short > 0:
                         if order_pct == 1.0:
-                            order_amount = current_position['available_short']
+                            order_amount = current_position.available_short
                         else:
-                            order_amount = math.floor(current_position['available_short'] * order_pct)
+                            order_amount = math.floor(current_position.available_short * order_pct)
 
                         if order_amount != 0:
                             self.update_position(current_position, order_amount, current_price, order_type,
                                                  current_timestamp)
                         else:
                             self.logger.warning("{} available_long:{} order_pct:{} order_amount:{}", entity_id,
-                                                current_position['available_long'], order_pct, order_amount)
+                                                current_position.available_long, order_pct, order_amount)
                     else:
                         raise Exception("not enough position")
