@@ -5,19 +5,16 @@ from typing import List, Union
 
 import pandas as pd
 
-from zvdata import IntervalLevel, EntityMixin
-from zvdata.contract import get_db_session
-from zvdata.normal_data import NormalData
-from zvdata.utils.time_utils import to_pd_timestamp, now_pd_timestamp
-from zvt.api.business import get_trader
-from zvt.api.common import get_one_day_trading_minutes
-from zvt.api.rules import iterate_timestamps, is_open_time, is_in_finished_timestamps, is_close_time, is_trading_date
-from zvt.domain import business, Stock, Coin
+from zvt.api.business_reader import AccountStatsReader
+from zvt.contract import IntervalLevel, EntityMixin
+from zvt.contract.api import get_db_session
+from zvt.contract.normal_data import NormalData
+from zvt.domain import Stock, TraderInfo, AccountStats
 from zvt.drawer.drawer import Drawer
 from zvt.factors.target_selector import TargetSelector
-from zvt.reader.business_reader import AccountReader
-from zvt.trader import TradingSignal, TradingSignalType
+from zvt.trader import TradingSignal, TradingSignalType, TradingListener
 from zvt.trader.account import SimAccountService
+from zvt.utils.time_utils import to_pd_timestamp, now_pd_timestamp, to_time_str
 
 logger = logging.getLogger(__name__)
 
@@ -95,13 +92,11 @@ class TargetsSlot(object):
 
 
 class Trader(object):
-    logger = logging.getLogger(__name__)
-
     entity_schema: EntityMixin = None
 
     def __init__(self,
                  entity_ids: List[str] = None,
-                 exchanges: List[str] = ['sh', 'sz'],
+                 exchanges: List[str] = None,
                  codes: List[str] = None,
                  start_timestamp: Union[str, pd.Timestamp] = None,
                  end_timestamp: Union[str, pd.Timestamp] = None,
@@ -110,16 +105,18 @@ class Trader(object):
                  trader_name: str = None,
                  real_time: bool = False,
                  kdata_use_begin_time: bool = False,
-                 draw_result: bool = True) -> None:
-
+                 draw_result: bool = True,
+                 solo: bool = False) -> None:
         assert self.entity_schema is not None
 
+        self.logger = logging.getLogger(__name__)
+
         if trader_name:
             self.trader_name = trader_name
         else:
             self.trader_name = type(self).__name__.lower()
 
-        self.trading_signal_listeners = []
+        self.trading_signal_listeners: List[TradingListener] = []
 
         self.selectors: List[TargetSelector] = []
 
@@ -139,6 +136,9 @@ class Trader(object):
         else:
             assert False
 
+        self.trading_dates = self.entity_schema.get_trading_dates(start_date=self.start_timestamp,
+                                                                  end_date=self.end_timestamp)
+
         if real_time:
             logger.info(
                 'real_time mode, end_timestamp should be future,you could set it big enough for running forever')
@@ -146,8 +146,10 @@ class Trader(object):
 
         self.kdata_use_begin_time = kdata_use_begin_time
         self.draw_result = draw_result
+        self.solo = solo
 
-        self.account_service = SimAccountService(trader_name=self.trader_name,
+        self.account_service = SimAccountService(entity_schema=self.entity_schema,
+                                                 trader_name=self.trader_name,
                                                  timestamp=self.start_timestamp,
                                                  provider=self.provider,
                                                  level=self.level)
@@ -159,27 +161,24 @@ class Trader(object):
 
         self.selectors_comparator = self.init_selectors_comparator()
 
-        self.trading_level_asc = list(set([IntervalLevel(selector.level) for selector in self.selectors]))
-        self.trading_level_asc.sort()
+        if self.selectors:
+            self.trading_level_asc = list(set([IntervalLevel(selector.level) for selector in self.selectors]))
+            self.trading_level_asc.sort()
 
-        self.trading_level_desc = list(self.trading_level_asc)
-        self.trading_level_desc.reverse()
+            self.logger.info(f'trader level:{self.level},selectors level:{self.trading_level_asc}')
+
+            if self.level != self.trading_level_asc[0]:
+                raise Exception("trader level should be the min of the selectors")
+
+            self.trading_level_desc = list(self.trading_level_asc)
+            self.trading_level_desc.reverse()
 
         self.targets_slot: TargetsSlot = TargetsSlot()
 
-        self.session = get_db_session('zvt', 'business')
-        trader = get_trader(session=self.session, trader_name=self.trader_name, return_type='domain', limit=1)
-
-        if trader:
-            self.logger.warning("trader:{} has run before,old result would be deleted".format(self.trader_name))
-            self.session.query(business.Trader).filter(business.Trader.trader_name == self.trader_name).delete()
-            self.session.commit()
+        self.session = get_db_session('zvt', data_schema=TraderInfo)
         self.on_start()
 
     def on_start(self):
-        if not self.selectors:
-            raise Exception('please setup self.selectors in init_selectors at first')
-
         # run all the selectors
         for selector in self.selectors:
             # run for the history data at first
@@ -200,14 +199,20 @@ class Trader(object):
         else:
             codes = None
 
-        trader_domain = business.Trader(id=self.trader_name, timestamp=self.start_timestamp,
-                                        trader_name=self.trader_name,
-                                        entity_type=entity_ids, exchanges=exchanges, codes=codes,
-                                        start_timestamp=self.start_timestamp,
-                                        end_timestamp=self.end_timestamp, provider=self.provider,
-                                        level=self.level.value,
-                                        real_time=self.real_time, kdata_use_begin_time=self.kdata_use_begin_time)
-        self.session.add(trader_domain)
+        sim_account = TraderInfo(id=self.trader_name,
+                                 entity_id=f'trader_zvt_{self.trader_name}',
+                                 timestamp=self.start_timestamp,
+                                 trader_name=self.trader_name,
+                                 entity_ids=entity_ids,
+                                 exchanges=exchanges,
+                                 codes=codes,
+                                 start_timestamp=self.start_timestamp,
+                                 end_timestamp=self.end_timestamp,
+                                 provider=self.provider,
+                                 level=self.level.value,
+                                 real_time=self.real_time,
+                                 kdata_use_begin_time=self.kdata_use_begin_time)
+        self.session.add(sim_account)
         self.session.commit()
 
     def init_selectors(self, entity_ids, entity_schema, exchanges, codes, start_timestamp, end_timestamp):
@@ -215,7 +220,7 @@ class Trader(object):
         implement this to init selectors
 
         """
-        raise NotImplementedError
+        pass
 
     def init_selectors_comparator(self):
         """
@@ -232,15 +237,16 @@ class Trader(object):
         if listener in self.trading_signal_listeners:
             self.trading_signal_listeners.remove(listener)
 
-    def handle_targets_slot(self, timestamp):
+    def handle_targets_slot(self, due_timestamp: pd.Timestamp, happen_timestamp: pd.Timestamp):
         """
         this function would be called in every cycle, you could overwrite it for your custom algorithm to select the
         targets of different levels
 
         the default implementation is selecting the targets in all levels
 
-        :param timestamp:
-        :type timestamp:
+        :param due_timestamp:
+        :param happen_timestamp:
+
         """
         long_selected = None
         short_selected = None
@@ -260,67 +266,92 @@ class Trader(object):
                 else:
                     short_selected = short_selected & short_targets
 
-        self.logger.debug('timestamp:{},long_selected:{}'.format(timestamp, long_selected))
+        self.logger.debug('timestamp:{},long_selected:{}'.format(due_timestamp, long_selected))
 
-        self.logger.debug('timestamp:{},short_selected:{}'.format(timestamp, short_selected))
+        self.logger.debug('timestamp:{},short_selected:{}'.format(due_timestamp, short_selected))
 
-        self.send_trading_signals(timestamp=timestamp, long_selected=long_selected, short_selected=short_selected)
+        self.trade_the_targets(due_timestamp=due_timestamp, happen_timestamp=happen_timestamp,
+                               long_selected=long_selected, short_selected=short_selected)
 
-    def send_trading_signals(self, timestamp, long_selected, short_selected):
+    def get_current_account(self) -> AccountStats:
+        return self.account_service.account
+
+    def buy(self, due_timestamp, happen_timestamp, entity_ids, position_pct=1.0):
+        if entity_ids:
+            position_pct = (1.0 / len(entity_ids)) * position_pct
+
+        for entity_id in entity_ids:
+            trading_signal = TradingSignal(entity_id=entity_id,
+                                           due_timestamp=due_timestamp,
+                                           happen_timestamp=happen_timestamp,
+                                           trading_signal_type=TradingSignalType.open_long,
+                                           trading_level=self.level,
+                                           position_pct=position_pct)
+            self.send_trading_signal(trading_signal)
+
+    def sell(self, due_timestamp, happen_timestamp, entity_ids, position_pct=1.0):
         # current position
-        account = self.account_service.latest_account
-        current_holdings = [position['entity_id'] for position in account['positions'] if
-                            position['available_long'] > 0]
+        account = self.get_current_account()
+        current_holdings = []
+        if account.positions:
+            current_holdings = [position.entity_id for position in account.positions if position != None and
+                                position.available_long > 0]
 
-        if long_selected:
-            # just long the security not in the positions
-            longed = long_selected - set(current_holdings)
-            if longed:
-                position_pct = 1.0 / len(longed)
-                order_money = account['cash'] * position_pct
+        shorted = set(current_holdings) & entity_ids
 
-                for entity_id in longed:
-                    trading_signal = TradingSignal(entity_id=entity_id,
-                                                   the_timestamp=timestamp,
-                                                   trading_signal_type=TradingSignalType.trading_signal_open_long,
-                                                   trading_level=self.level,
-                                                   order_money=order_money)
-                    for listener in self.trading_signal_listeners:
-                        listener.on_trading_signal(trading_signal)
+        if shorted:
+            position_pct = (1.0 / len(shorted)) * position_pct
 
-        # just short the security in current_holdings and short_selected
-        if short_selected:
-            shorted = set(current_holdings) & short_selected
+        for entity_id in shorted:
+            trading_signal = TradingSignal(entity_id=entity_id,
+                                           due_timestamp=due_timestamp,
+                                           happen_timestamp=happen_timestamp,
+                                           trading_signal_type=TradingSignalType.close_long,
+                                           trading_level=self.level,
+                                           position_pct=position_pct)
+            self.send_trading_signal(trading_signal)
 
-            for entity_id in shorted:
-                trading_signal = TradingSignal(entity_id=entity_id,
-                                               the_timestamp=timestamp,
-                                               trading_signal_type=TradingSignalType.trading_signal_close_long,
-                                               position_pct=1.0,
-                                               trading_level=self.level)
-                for listener in self.trading_signal_listeners:
-                    listener.on_trading_signal(trading_signal)
+    def trade_the_targets(self, due_timestamp, happen_timestamp, long_selected, short_selected, long_pct=1.0,
+                          short_pct=1.0):
+        self.buy(due_timestamp=due_timestamp, happen_timestamp=happen_timestamp, entity_ids=long_selected,
+                 position_pct=long_pct)
+        self.sell(due_timestamp=due_timestamp, happen_timestamp=happen_timestamp, entity_ids=short_selected,
+                  position_pct=short_pct)
+
+    def send_trading_signal(self, signal: TradingSignal):
+        for listener in self.trading_signal_listeners:
+            try:
+                listener.on_trading_signal(signal)
+            except Exception as e:
+                self.logger.exception(e)
+                listener.on_trading_error(timestamp=signal.happen_timestamp, error=e)
 
     def on_finish(self):
         # show the result
         if self.draw_result:
             import plotly.io as pio
             pio.renderers.default = "browser"
-            reader = AccountReader(trader_names=[self.trader_name])
-            df = reader.data_df.reset_index()
+            reader = AccountStatsReader(trader_names=[self.trader_name])
+            df = reader.data_df
             drawer = Drawer(main_data=NormalData(df.copy()[['trader_name', 'timestamp', 'all_value']],
                                                  category_field='trader_name'))
-            drawer.draw_line()
+            drawer.draw_line(show=True)
+
+    def in_trading_date(self, timestamp):
+        return to_time_str(timestamp) in self.trading_dates
+
+    def on_time(self, timestamp):
+        self.logger.debug(f'current timestamp:{timestamp}')
 
     def run(self):
         # iterate timestamp of the min level,e.g,9:30,9:35,9.40...for 5min level
         # timestamp represents the timestamp in kdata
-        for timestamp in iterate_timestamps(entity_type=self.entity_schema, exchange=self.exchanges[0],
-                                            start_timestamp=self.start_timestamp, end_timestamp=self.end_timestamp,
-                                            level=self.level):
+        for timestamp in self.entity_schema.get_interval_timestamps(start_date=self.start_timestamp,
+                                                                    end_date=self.end_timestamp, level=self.level):
 
-            if not is_trading_date(entity_type=self.entity_schema, exchange=self.exchanges[0], timestamp=timestamp):
+            if not self.in_trading_date(timestamp=timestamp):
                 continue
+
             if self.real_time:
                 # all selector move on to handle the coming data
                 if self.kdata_use_begin_time:
@@ -328,44 +359,44 @@ class Trader(object):
                 else:
                     real_end_timestamp = timestamp
 
-                waiting_seconds, _ = self.level.count_from_timestamp(real_end_timestamp,
-                                                                     one_day_trading_minutes=get_one_day_trading_minutes(
-                                                                         entity_type=self.entity_schema))
+                seconds = (now_pd_timestamp() - real_end_timestamp).total_seconds()
+                waiting_seconds = self.level.to_second() - seconds,
                 # meaning the future kdata not ready yet,we could move on to check
-                if waiting_seconds and (waiting_seconds > 0):
+                if waiting_seconds > 0:
                     # iterate the selector from min to max which in finished timestamp kdata
                     for level in self.trading_level_asc:
-                        if (is_in_finished_timestamps(entity_type=self.entity_schema, exchange=self.exchanges[0],
-                                                      timestamp=timestamp, level=level)):
+                        if self.entity_schema.is_finished_kdata_timestamp(timestamp=timestamp, level=level):
                             for selector in self.selectors:
                                 if selector.level == level:
                                     selector.move_on(timestamp, self.kdata_use_begin_time, timeout=waiting_seconds + 20)
 
             # on_trading_open to setup the account
             if self.level == IntervalLevel.LEVEL_1DAY or (
-                    self.level != IntervalLevel.LEVEL_1DAY and is_open_time(entity_type=self.entity_schema,
-                                                                            exchange=self.exchanges[0],
-                                                                            timestamp=timestamp)):
+                    self.level != IntervalLevel.LEVEL_1DAY and self.entity_schema.is_open_timestamp(timestamp)):
                 self.account_service.on_trading_open(timestamp)
 
-            # the time always move on by min level step and we could check all level targets in the slot
-            self.handle_targets_slot(timestamp=timestamp)
+            if self.solo:
+                self.on_time(timestamp=timestamp)
 
-            for level in self.trading_level_asc:
-                # in every cycle, all level selector do its job in its time
-                if (
-                is_in_finished_timestamps(entity_type=self.entity_schema.__name__.lower(), exchange=self.exchanges[0],
-                                          timestamp=timestamp, level=level)):
-                    long_targets, short_targets = self.selectors_comparator.make_decision(timestamp=timestamp,
-                                                                                          trading_level=level)
+            if self.selectors:
+                for level in self.trading_level_asc:
+                    # in every cycle, all level selector do its job in its time
+                    if self.entity_schema.is_finished_kdata_timestamp(timestamp=timestamp, level=level):
+                        long_targets, short_targets = self.selectors_comparator.make_decision(timestamp=timestamp,
+                                                                                              trading_level=level)
 
-                    self.targets_slot.input_targets(level, long_targets, short_targets)
+                        if long_targets or short_targets:
+                            self.targets_slot.input_targets(level, long_targets, short_targets)
+                            # the time always move on by min level step and we could check all level targets in the slot
+                            # 1)the targets is generated for next interval
+                            # 2)the acceptable price is next interval prices,you could buy it at current price if the time is before the timestamp(order_timestamp) when trading signal received
+                            # 3)the suggest price the the close price for generating the signal(generated_timestamp)
+                            due_timestamp = timestamp + pd.Timedelta(seconds=self.level.to_second())
+                            self.handle_targets_slot(due_timestamp=due_timestamp, happen_timestamp=timestamp)
 
             # on_trading_close to calculate date account
             if self.level == IntervalLevel.LEVEL_1DAY or (
-                    self.level != IntervalLevel.LEVEL_1DAY and is_close_time(entity_type=self.entity_schema,
-                                                                             exchange=self.exchanges[0],
-                                                                             timestamp=timestamp)):
+                    self.level != IntervalLevel.LEVEL_1DAY and self.entity_schema.is_close_timestamp(timestamp)):
                 self.account_service.on_trading_close(timestamp)
 
         self.on_finish()
@@ -373,7 +404,3 @@ class Trader(object):
 
 class StockTrader(Trader):
     entity_schema = Stock
-
-
-class CoinTrader(Trader):
-    entity_schema = Coin
